<?php

class Router
{
	protected static $routes = array();
	protected static $whitelistRegex;
	protected static $files = array('css' => 'text/css', 'jpg' => 'image/jpeg', 'png' => 'image/png');

	public static function addRoute($regex, $params)
	{
		self::$routes[$regex] = $params;
	}

	public static function setWhitelistRegex($regex)
	{
		self::$whitelistRegex = $regex;
	}

	/**
	 * Routes a URL.
	 * 
	 * @param url Url to route
	 */
	public static function route($url)
	{
		$url = preg_replace('/\/$/', '', $url, 1); //Remove / from the end of a URL

		if (self::_isWhitelist($url))
			self::_routeWhitelist($url);
		$arguments = explode('/', $url);
		$urlAction = array_shift($arguments);
		
		$arguments['action'] = $urlAction;
		if ($urlAction == 'admin')
			$arguments['action'] = array_shift($arguments);
		$arguments['url'] = $url;

		//$parameters = @self::$routes[$urlAction];
		$parameters = self::_findMatch($url);

		if (!$parameters)
			throw new Exception('I\'ve been told to do nothing by config/routes.php!<br />Url action: ' . $urlAction);

		Router::_invoke($parameters['controller'], $parameters['action'], $arguments);
	}
	
	public static function baseUrl()
	{
		$name = $_SERVER['SERVER_NAME'];
		//$url = $_SERVER['REQUEST_URI'];
		$url = $_SERVER['REDIRECT_URL']; //Looks better
		
		$location = strpos($url, $_GET['url']);
		return substr($url, 0, $location);
	}

	protected static function _invoke($controller, $action, $arguments)
	{
		//$template = str_replace('_', '/', $action);
		$template = strtolower($controller) . '/' . str_replace('_', '/', $action);
		
		/*echo '<pre>Controller: ' . $controller . '</pre>';
		echo '<pre>Action: ' . $action . '</pre>';
		echo '<pre>Template: ' . $template . '</pre>';
		echo '<pre>Arguments: ' . print_r($arguments, true) . '</pre>';*/
		
		$file = strtolower('controllers/' . $controller . '.php');
		if (!is_file($file))
			throw new Exception($file . ' not found!');
		require_once($file);
		
		$c = new $controller();
		if (!is_subclass_of($c, 'Controller'))
			throw new Exception($file . ' does not extend Controller!');
		
		$c->__beforeAction();
		
		if (!method_exists($c, $action))
			throw new Exception('Function ' . $action . '() not found in ' . $controller . ' controller!');
		$c->$action($arguments);
		
		if ($c->isAutoRender()) {
			$c->setTemplate($template);
			$c->render();
		}
	}
	
	protected static function _isWhitelist($path)
	{
		return self::_match(self::$whitelistRegex, $path);
	}
	
	protected static function _routeWhitelist($path)
	{
		$file = WWW_DIR.'/'.$path;
		
		if (!is_file($file))
			die('404');
		
		foreach (self::$files as $ending => $mimeType)
			if (strrpos($file, '.'.$ending) === strlen($file) - strlen('.'.$ending))
				header('Content-type: ' . $mimeType);

		/*header('Content-type: application/octet-stream');
		header('Content-Length: ' . filesize($file));*/
		echo readfile($file);
		exit;
	}
	
	protected static function _findMatch($path)
	{
		foreach (self::$routes as $regex => $route)
			if (self::_match(self::prepareRouteRegex($regex), $path))
				return $route;
		return false;
	}
	
	private static function prepareRouteRegex($regex)
	{
		$replace = array(
			'/' => '\/'
		);
		
		foreach ($replace as $needle => $replacement)
			$regex = str_replace($needle, $replacement, $regex);
		
		return '^' . $regex . '$';
	}
	
	public static function _match($regex, $path)
	{
		return preg_match('/'.$regex.'/', $path) != 0;
	}
}